# frozen_string_literal: true
require 'json'
require 'net/http'

# checker Api::Dmo::HermesController avant d'ajouter des methodes
class Hermes::Api::Dmo::HousingRequestsController < Hermes::Api::Dmo::ApplicationController
  hermes_tools 'main_user_id'
  hermes_errors 'housing_channel_error'

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $regex
  ]

  ACCEPTABLE_PARAMS = %w[
    applicant_used_for_siret_firstname
    applicant_used_for_siret_lastname
    applicant_used_for_siret_fullname
    applicant_used_for_siret_email
    applicant_used_for_siret_birth_date
    created_at
    updated_at
    unique_numbers
    backoffice_status
    viewed_by_colaborator_offre_logement
    form_status
    ids
    als_id
    is_active
    request_status
    attributed_housing_district
    attributed_housing_district_postal_code
    old_request_status
    request_status_updated_at
    old_entity_status
    updated_entity_status
    entity_status
    dalo
    asked_mobility
    valid_mobility
    decider
    submitted_to_cal_nb
    sent_to_cal_nb
    rejected_in_cal_nb
    rejected_after_cal_nb
    accepted_in_cal_nb
    mobility
    seeked_types
    entity_help_required
    contract_type
    registration_number
    priority_points_indexes
    nud
    nb_of_requests_made
    reference_tax_revenues
    seeked_districts
    category_type
    single_parent
    id_counter
    id_unique
    id_unique_associate
    internal_ref
    forcing_duplicate
    electronical_renewal
    ec_law_ressources
    is_long_delay_anormal
    date_calculator_indicator
    with_car_park
    widen_request
    housing_type_list
    expulsion_judgement_date
    other_reason
    work_contract_type
    seeked_district_postal_codes
    seeked_departments
    seeked_localisation_list
    dependents
    monthly_revenues
    typology_range
    entity_validated
    entity_refused
    coapplicants
    lessor_name
    lessor_siren
    social_housing_entry_date
    social_housing_name
    test_created_at
    sne_number
    employee_id
    employee_start_date
    entity_access_level
    employee_unit
    tab_state
    motivations
    seeking_first_floor
    seeking_elevator
    seeked_max_rent
    sne_created_at
    precisions
    has_multiple_attributions_under_conditions
    seeked_housing_categories
    min_surface
    main_applicant_id
    main_applicant
    main_employer_siret
    requester_firstname
    requester_lastname
    requester_fullname
    requester_birth_date
    entity_id
    coapplicant_ids
    coapplicants
    seeked_epci_first_quartils
    seeked_quartil_belongings
    dependents_monthly_incomes
    dependents_monthly_charges
    housing_offer_ids
    documents_required
    documents_uploaded
    last_application_date
    is_house_owner
    disability_details
    unborn_children
    owned_housing_details
    actual_housing_details
    postal_details
    has_priority
    available
    is_untensed_area
    last_entity_code
    cdii
    last_update_user_id
    last_update_user_last_name
    last_update_user_first_name
    housing_channel_keys
    consent_level
    profile_score
    is_striked_off
    cotations
    business_cotation
    target_offer_reserving_business
    applicants_attached_codes
    cerfa_version
  ]

  REGEX_PARAMS = %w[
    requester_firstname
    requester_lastname
    requester_fullname
    applicant_used_for_siret_firstname
    applicant_used_for_siret_lastname
    applicant_used_for_siret_fullname
  ]

  MUTATE_PARAMS = %i[
    backoffice_status
    viewed_by_colaborator_offre_logement
    form_status
    workplace_insee_code
    is_active
    attributed_housing_district
    attributed_housing_district_postal_code
    asked_mobility
    valid_mobility
    decider
    mobility
    seeked_types
    entity_help_required
    contract_type
    registration_number
    priority_points_indexes
    nud
    id_counter
    id_unique
    id_unique_associate
    internal_ref
    forcing_duplicate
    electronical_renewal
    ec_law_ressources
    is_long_delay_anormal
    date_calculator_indicator
    with_car_park
    widen_request
    housing_type_list
    expulsion_judgement_date
    other_reason
    work_contract_type
    seeked_localisation_list
    dependents
    lessor_name
    lessor_siren
    social_housing_entry_date
    social_housing_name
    test_created_at
    sne_number
    employee_id
    employee_start_date
    entity_access_level
    employee_unit
    tab_state
    motivations
    seeking_first_floor
    seeking_elevator
    seeked_max_rent
    sne_created_at
    last_sne_renewal_date
    precisions
    has_multiple_attributions_under_conditions
    seeked_housing_categories
    main_applicant_id
    coapplicants
    seeked_epci_first_quartils
    seeked_quartil_belongings
    dependents_monthly_incomes
    dependents_monthly_charges
    submitted_to_cal_nb
    sent_to_cal_nb
    rejected_in_cal_nb
    rejected_after_cal_nb
    accepted_in_cal_nb
    documents_required
    documents_uploaded
    is_house_owner
    disability_details
    unborn_children
    owned_housing_details
    actual_housing_details
    postal_details
    body
    channels
    cdii
    applicant_used_for_siret
    business_cotation
    entity_validated
    entity_refused
  ]

  def self.acceptable_params
    ACCEPTABLE_PARAMS
  end 

  def self.regex_params
    REGEX_PARAMS
  end 
  
  def mutate_params
    MUTATE_PARAMS
  end 

  def include_relationships
    [:housing_applications]
  end

  def allow_true_user
    @allow_true_user = true
  end

  def regex_fullword_without_case_accent(param)
    if param && param[:$regex]
      param[:$regex] = /^#{::Regexp.escape(param[:$regex]).accent_to_regexp}/i
    end
  end

  def search
    index
  end

  def index
    acceptable_params *ACCEPTABLE_PARAMS
    regex_params *REGEX_PARAMS
    valid_operators *VALID_OPERATORS
    @query_precision ||= {}

    regex_fullword_without_case_accent(params[:requester_firstname])
    regex_fullword_without_case_accent(params[:requester_lastname])
    regex_fullword_without_case_accent(params[:applicant_used_for_siret_firstname])
    regex_fullword_without_case_accent(params[:applicant_used_for_siret_lastname])
    # It return only HousingRequest linked to current lessor (with params available = true)
    if current_lessor && params[:available].to_bool
      def include_relationships
        []
      end

      request_ids = Hermes::HousingApplication.where(
        :housing_offer_id.in => current_lessor.housing_offer_ids,
        is_active: true
      ).pluck(:housing_request_id).map(&:to_s)

      @query_precision[:id] = {'$in': request_ids}
    end
    if params[:main_employer_siret]
      applicants_ids = Hermes::Applicant.only(:id, :main_employer_siret).where(:main_employer_siret => {'$in' => params[:main_employer_siret]}).pluck(:id)
      @query_precision[:main_applicant_id] = {'$in': applicants_ids}
    end

    aggregation = index_aggregation

    if params[:requester_email]
      aggregation = aggregation.where(requester_email: /^#{Regexp.escape(params[:requester_email])}/i)
    end
    
    paginate(
      aggregation,
      each_serializer: @serializer,
      **@options
    )
  end

  def entity_requests
    housing_request_id = params[:id]
    housing_request = Hermes::HousingRequest.find(housing_request_id)
    entity_requests_account = housing_request.entity_request_account
    render json: {status: "success", code: 200, message: {'entity_requests_account' => entity_requests_account}}
  end

  def spent_points
    housing_request_id = params[:id]
    housing_request = Hermes::HousingRequest.find(housing_request_id)
    entities_requests = housing_request.entity_request_account
    rented_requests = entities_requests.select {|request| request.request_rented == true}
    unless rented_requests.nil?
      spent_points = sum(rented_requests)
      render json: {status: "success", code: 200, message: {'spent_points' => spent_points}}
    end
  end

  def sum(requests)
    sum = 0
    requests.each do |request|
      sum += request["amount"].to_i
    end
    sum
  end

  def waiting_points
    housing_request_id = params[:id]
    housing_request = Hermes::HousingRequest.find(housing_request_id)
    entities_requests = housing_request.entity_request_account
    rented_requests = entities_requests.select {|request| request.request_rented == false}
    unless rented_requests.nil?
      waiting_points = sum(rented_requests)
      render json: {status: "success", code: 200, message: {'waiting_points' => waiting_points}}
    end
  end

  def is_a_collaborator_company
    current_user_roles.include?('collaborateur entreprise')
  end

  def is_a_collaborator_housing_offer
    current_user_roles.include?('collaborateur offre logement')
  end

  def entity_readable(housing_request)
    if Hermes::Entity.readable_by(@user).find_by(id: housing_request.entity.id)
      true
    else
      @error = " entity not readable by user"
      false
    end
  end

  def has_a_company_role
    if current_user_roles.include?('entreprise')
      true
    else
      @error = " user role doesn't include entreprise"
    end
  end

  def housing_request_match(housing_request)
    if housing_request.entity.id.to_s == params[:entity_id]
      true
    else
      @error = " housing request is not connected to this entity"
      false
    end
  end

  def has_an_entity(housing_request)
    if !housing_request.entity.nil?
      true
    else
      @error = "housing request has no entity"
      false
    end
  end

  def add_points_to_housing_request
    if !params[:points].nil? && !params[:entity_id].nil?
      if housing_request = Hermes::HousingRequest.find(params[:id])
        if has_an_entity(housing_request) && has_a_company_role && housing_request_match(housing_request) #&& entity_readable
          points = params[:points].to_i
          entity = Hermes::Entity.find_by(id: params[:entity_id])
          response_json = entity.create_entity_request_account(points, housing_request)
          render json: response_json
        else
          render json: {status: "unauthorized", code: 401, message: @error}
        end
      else
        render json: {status: "not found", code: 404, message: 'housing request not found'}
      end
    else
      render json: {status: "missing params", code: 401, message: 'some params are missing'}
    end
  end

  def entity_note
    set_instance
    if @instance
      if has_an_entity(@instance) && entity_readable(@instance) && current_user_roles.include?('cre')
        if @instance.entity_note.nil?
          render json: {status: "no content", code: 204, message: 'no entity note inside the housing request'}
        else
          render json: {status: "success", code: 200, message: @instance.entity_note}
        end
      else
        render json: {status: "unauthorized", code: 401, message: @error}
      end
    else
      render json: {status: "not found", code: 404, message: 'no housing request found'}
    end
  end

  def internal_note
    housing_request_id = params[:id]
    if !housing_request_id.nil?
      if is_a_collaborator_housing_offer
        housing_request = Hermes::HousingRequest.find(housing_request_id)
        if housing_request.nil?
          render json: {status: "not found", code: 404, message: 'no housing request found'}
        else
          if housing_request.internal_note.nil?
            render json: {status: "no content", code: 204, message: 'no internal note inside the housing request'}
          else
            render json: {status: "success", code: 200, message: housing_request.internal_note}
          end
        end
      else
        render json: {status: "unauthorized", code: 401, message: 'access denied user is no collaborator'}
      end
    else
      render json: {status: "missing param", code: 404, message: 'param is missing'}
    end
  end

  def update_entity_note
    set_instance
    new_entity_note = params[:entity_note]
    if !new_entity_note.nil?
      if @instance.nil?
        render json: {status: "not found", code: 404, message: 'no housing request found'}
      else
        if has_an_entity(@instance) && has_a_company_role
          sentence = bad_words_filter(new_entity_note)
          @instance.entity_note = sentence
          if @instance.save!
            render json: {status: "success", code: 200, message: {"message" => "successfullly modified", "value" => @instance.entity_note}}
          else
            render json: {status: "no content", code: 204, message: 'no internal note inside the housing request'}
          end
        end
      end
    else
      render json: {status: "missing params", code: 404, message: 'some params are missing'}
    end
  end

  def update_internal_note
    housing_request_id = params[:id]
    new_internal_note = params[:internal_note]
    if !housing_request_id.nil? && !new_internal_note.nil?
      if is_a_collaborator_housing_offer
        housing_request = Hermes::HousingRequest.find(housing_request_id)
        if housing_request.nil?
          render json: {status: "not found", code: 404, message: 'no housing request found'}
        else
          sentence = bad_words_filter(new_internal_note)
          housing_request.internal_note = sentence
          if housing_request.save!
            render json: {status: "success", code: 200, message: {"message" => "successfullly modified", "value" => housing_request.internal_note}}
          else
            render json: {status: "no content", code: 204, message: 'no internal note inside the housing request'}
          end
        end
      else
        render json: {status: "unauthorized", code: 401, message: 'access denied user is no collabroator'}
      end
    else
      render json: {status: "missing params", code: 404, message: 'some params are missing'}
    end
  end

  def bad_words_filter(sentence)
    filters = ["fuck", "fucking"] #to link with the model
    words = sentence.split(' ')
    words.size.times do |i|
      filters.each do |filter|
        clean_word = words[i].split(/\W+/).first
        if filter == clean_word
          words[i] = "***"
        end
      end
    end
    words.join(' ')
  end

  def has_application
    @instance.housing_offer_ids.kind_of?(Array) && @instance.housing_offer_ids.length > 0
  end

  def set_instance
    if params[:id] == 'active'
      if main_applicant_id = @user.main_applicant_id
        @instance = @model.readable_by(@user).find_by(main_applicant_id: main_applicant_id, is_active: true)
      end
    else
      instance = @model.find_by(id: params[:id], **@query_precision) if params[:id]
      if (instance && (instance.readable_by?(@user) || instance&.entity&.readable_by?(@user)))
        @instance = instance
      else
        nil
      end
    end
  end

  def show
    return render_json_error(:not_found, :object_not_found) if !@instance
    super
  end

  def zip
    return render_json_error(:not_found, :object_not_found) if !@instance
    filename = "dossier_documents_hermes_#{@instance.id}_#{Time.current.to_i}.zip"
    folder = @instance.housing_request_folder
    return render_json_error(:not_found, :object_not_found) unless folder
    doc_ids = folder.required_documents.readable_by(@user).pluck(:doc_id)
    docs = Doc.where(:id.in => doc_ids, type: 'file')
    send_files_as_zip(docs, filename)
  end

  def create
    @model.current_user_id = @user.id
    @additional_params = {main_user_id: main_user_id_param} if main_user_id_param
    # return render_error_not_mutable(not_mutable) unless not_mutable.empty?

    create_without_render
    update_last_updated_user_info

    return respond_with_errors(@instance) unless @instance.save

    render(jsonapi: @instance, serializer: @serializer, status: :created)
  end

  def update_last_updated_user_info
    @instance.last_update_user_id = @user.id.to_s
    @instance.last_update_user_last_name = @user.last_name
    @instance.last_update_user_first_name = @user.first_name
  end

  def update
    @model.current_user_id = @user.id
    @additional_params = {main_user_id: main_user_id_param} if main_user_id_param

    return render_json_error(:not_found, :object_not_found) if !@instance 
    # return render_error_not_mutable(not_mutable) unless not_mutable.empty?
    return render_json_error(:unprocessable_entity, :desactivation_avec_candidature) if mutate_valid_params[:is_active] === false && has_application
    return render_json_error(:unprocessable_entity, :cannot_validate) if params[:data][:attributes][:entity_validated].present? && !@instance.entity&.writable_by?(@user)
    
    @instance.with_lock(reload: false) do
      valid_params = mutate_valid_params
      
      param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
      valid_params[:user_id] = param_user if param_user && @user.has_role?('Administrator')
      return render_json_error(:parameters_not_present, :no_valid_parameters) unless valid_params.present?
      Rails.logger.debug "TEST valid_params are #{valid_params}"
      update_last_updated_user_info
      if @instance.update(valid_params) && @instance.save
        # Temporary_disabled_devops_7390
        # @instance.notify(key: NOTIFICATION_KEYS[:housing_request][:al_edit], sender: @user, channels: [])
      else
        return respond_with_errors(@instance)
      end

      render(jsonapi: @instance, serializer: @serializer)
    end
  end

  def update_next_renewal_date_set_by_sne
    return render_json_error(:unauthorized, :not_admin) unless current_user_roles.include?('Administrator') || is_super_admin
    return render_json_error(:not_found, :object_not_found) unless @instance 
    return render_json_error(:unprocessable_entity, :missing_params) unless params[:next_renewal_date_set_by_sne].present?

    @instance.update(next_renewal_date_set_by_sne: params[:next_renewal_date_set_by_sne] )

    render(jsonapi: @instance, serializer: @serializer)
  end 

  def update_from_sne
    hri = nil
    return render_json_error(:not_found, :object_not_found) if !@instance
    return render_json_error(:unprocessable_entity, :trying_to_update_an_inactive_request) unless @instance.is_active
    unless @instance.applicant_verified
      hri = Hermes::HousingRequestInitializer.where(done: false, unique_number: @instance.unique_numbers.last).order_by(created_at: :desc).first
      return render_json_error(:not_found, :housing_request_initilizer_not_found) unless hri
      return render_json_error(:unprocessable_entity, :pin_code_missing) if params["pin_code"].blank? 
      return render_json_error(:unprocessable_entity, :wrong_pin_code) unless hri.check_pin_code(params["pin_code"])
      @instance.set(applicant_verified: true, updated_at: DateTime.now)
    end 

    @model.current_user_id = @user.id
    return render_json_error(:not_found, :object_not_found) if !@instance

    result = @instance.update_request_from_sne(current_user) 

    if result.is_a? Hash
      render json: result, status: 422 # a voir comment utilsier render_json_error pour donner une liste d'erreur
    else 
      render(
        jsonapi: @instance.reload,
        serializer: @serializer,
        **@options
      )
    end
  end

  def force_update_renewal_values
    return render_json_error(:not_found, :object_not_found) unless @instance 

    result = @instance.update_renewal_values_from_sne(current_user) 

    if result.is_a? Hash
      render json: result, status: 422 
    else 
      render(
        jsonapi: @instance.reload,
        serializer: @serializer,
        **@options
      )
    end
  end

  def destroy
    return render_json_error(:not_found, :object_not_found) if !@instance
    return render_json_error(:unprocessable_entity, :suppression_avec_candidature) if has_application

    super
  end

  def admin_search
    return render json: {status: "unauthorized", code: 401, message: "user is no superadmin"} unless is_super_admin

    entity_query = admin_search_param(params['entity_params'], 'entity', [:id, :id_cristal, :siren, :siret, :als_id], [], [:title])
    applicant_query = admin_search_param(params['applicant_params'], nil, [:id, :requester_email], [:requester_firstname, :requester_lastname])

    query = [
      {"$match": {is_active: true}},
      {"$match": applicant_query},
      {
        "$lookup": {
          from: 'hermes_entities',
          localField: "entity_id",
          foreignField: "_id",
          as: "entity"
        }
      },
      {"$unwind": {path: "$entity", preserveNullAndEmptyArrays: true}},
      {"$match": entity_query},
    ]

    if params[:unique_numbers].present?
      unique_numbers = params[:unique_numbers].is_a?(Array) ? params[:unique_numbers] : [params[:unique_numbers]]
      query.first[:$match][:unique_numbers] = { :$in => unique_numbers }
    end

    aggregation = Hermes::HousingRequest.collection.aggregate(query.push(facet_count_query(params)))

    mapped_attributes = {
      id: "['_id']",
      unique_numbers: "['unique_numbers']",
      als_id: "['als_id']",
      user_id: "['user_id']",
      applicant_id: "['main_applicant_id']",
      applicant_email: "['requester_email']",
      applicant_first_name: "['requester_firstname']",
      applicant_last_name: "['requester_lastname']",
      entity_id: "['entity_id']",
      entity_als_id: "['entity']['als_id']",
      entity_title: "['entity']['title']",
      entity_siret: "['entity']['siret']",
      entity_siren: "['entity']['siren']",
      entity_id_cristal: "['entity']['id_cristal']",
    }

    paginate_like_array(aggregation, mapped_attributes)
  end

  def clp_details
    return render_json_error(:missing_parameter, :als_ids_param_is_missing) unless params[:als_ids].present?

    aggregation = index_aggregation

    @options = { detailed_eligibility: true }
    
    if params[:reserving_siren].present?
      @options[:reserving_sirens] =  Hermes::ReservationHolder.find_by(siren: params[:reserving_siren])&.siren
    end 

    paginate(
      aggregation.where(:als_id.in => (params[:als_ids] || [])),
      each_serializer: @serializer,
      **@options
    )
  end

  def housing_request_folder
    render_json_error(:not_found, :object_not_found) unless @instance
    hrf = @instance.housing_request_folder
    #Rails.cache.fetch(hrf.cache_key, expires_in: 1.days) do
      render(
        jsonapi: hrf,
        serializer: Hermes::HousingRequestFolderSerializer,
        base_url: request.host,
        **@options
      )
    #end
  end


  def change_confirmed
    return render_json_error(:not_found, :object_not_found) unless @instance
    return render_json_error(:unprocessable_entity, :missing_params) if params[:required_documents].nil? || !params[:required_documents].is_a?(Hash)

    params[:required_documents].each do |key, value|
      if key == "all"
        @instance.housing_request_folder.required_documents.each do |rd|
          rd.update(declared: value)
        end 
        break
      end 
      Hermes::RequiredDocument.find(key)&.update(declared: value)
    end 

    render(
      jsonapi: @instance.housing_request_folder.reload,
      serializer: Hermes::HousingRequestFolderSerializer,
      **@options
    )
  end 

  def send_flux_thirty  
    array_of_ids = params[:ids].to_array
    array_of_als_ids = params[:als_ids].to_array
    return render_json_error(:unprocessable_entity, :wrong_params) if (array_of_ids.length + array_of_als_ids.length).zero?
    total = 0
    ids = array_of_ids
    cond = {}
    if ids.length > 0
      cond[:id.in] = ids
    else
      ids = array_of_als_ids
      cond[:als_id.in] = ids
    end
    ids.each do |id|
      if hr = Hermes::HousingRequest.find_by(cond)
        if hr.nuds && hr.nuds.length > 0
          hr.nuds.each do |nud|
            if nud.unique_number.length == 18
              total += 1
              so = Hermes::BoClient::AddRequestTask.new(housing_request_id: hr.id, nud_id: nud.id)
            end
          end
        elsif hr
          so = Hermes::BoClient::AddRequestTask.new(housing_request_id: hr.id)
        end
        if so
          total += 1
          ExecuteSopraTaskJob.perform_later(so.endpoint, so.to_xml, "post", "Hermes::HousingRequest", hr.id.to_s)
        end
      end
    end
    render body: {res: total.to_s + " housing_requests flux 30 sended to the workers"}.to_json, status: 200
  end

  def send_flux_thirty_sync
    if @instance
      if @instance.nuds && @instance.nuds.length > 0
        nud = @instance.nuds[0]
        if nud.unique_number.length == 18
          so = Hermes::BoClient::AddRequestTask.new(housing_request_id: @instance.id, nud_id: nud.id)
        end
      else
        so = Hermes::BoClient::AddRequestTask.new(housing_request_id: @instance.id)
      end
      if so
        res = ExecuteSopraTaskJob.perform_now(so.endpoint, so.to_xml, "post", "Hermes::HousingRequest", @instance.id.to_s)
      end
    end
    if res
      render body: {res: " housing_requests flux 30 sended with no errors"}.to_json, status: 200
    else
      if @instance.reload.xml_sopra_error_ids.last
      error = Hermes::XmlSopraError.find(@instance.reload.xml_sopra_error_ids.last)
      else
        error = "impossible de récuperer l'erreur"
        end
      render body: {error: error&.as_json}.to_json, status: 200
    end
  end

  def add_housing_channel
    keys = params[:keys] || [params[:key]]
    # If an array of keys is given, delete all existing keys to remplace by array
    if params[:keys].present?
      @instance.set(housing_channel_keys: [], updated_at: DateTime.current)
    end
    update_channel do
      keys.each do |key|
        if Hermes::HousingChannel.find_by(key: key)
          @instance.update_pending_channels!(key)
        else
          render_json_error(:not_found, :instance_not_found, {message: I18n.t('errors.housing_channel.not_found', key: key)}) and return
        end
      end
    end
  end

  def validate_housing_channel
    keys = params[:keys] || [params[:key]]
    update_channel do
      keys.each do |key|
        if Hermes::HousingChannel.find_by(key: key)
          @instance.update_validated_channels!(key)
        else
          render_json_error(:not_found, :instance_not_found, {message: I18n.t('errors.housing_channel.not_found', key: key)}) and return
        end
      end
    end
  end

  def remove_housing_channel
    keys = params[:keys] || [params[:key]]
    update_channel do
      keys.each do |key|
        if Hermes::HousingChannel.find_by(key: key)
          @instance.update_validated_channels!(key, false)
        else
          render_json_error(:not_found, :instance_not_found, {message: I18n.t('errors.housing_channel.not_found', key: key)}) and return
        end
      end
    end
  end

  def validate_code
    return render_json_error(:not_found, :instance_not_found) unless @instance
    return render_json_error(:not_acceptable, :missing_code) unless params.key?("code")

    if params[:code].nil?
      @instance.update(last_entity_code: nil)
      return render(jsonapi: @instance.reload, serializer: @serializer)
    end 

    entity_code = Hermes::EntityCode.find_by(code: params[:code], applicant_id: @instance.main_applicant_id)
    entity_code = Hermes::EntityCode.find_by(code: params[:code]) unless entity_code
    return render(json: {success: false, message: "Le code n'est pas valide."}, status: :not_found) unless entity_code

    if !entity_code.expired?
      applicant = Hermes::Applicant.find_by(id: @instance.main_applicant_id)
      return render_json_error(:not_found, :instance_not_found) unless applicant
      entity = Hermes::Entity.find(entity_code.entity_id)
      @instance.update(last_entity_code: entity_code.code)
      render(jsonapi: @instance.reload, serializer: @serializer)
    else
      render json: {
        success: false,
        message: 'Le code est expiré.'
      }, status: :unprocessable_entity
    end
  rescue ActionController::ParameterMissing => e
    render json: {success: false, message: e.message}, status: :not_acceptable
  end

  def notify
    return render_json_error(:not_found, :instance_not_found) unless @instance

    # mutate_valid_params include body and title (can includes channels array to)
    @notif = NotificationCenter::Notification.new(mutate_valid_params.merge({
                                                                              notifiable: @instance,
                                                                              targetable: @instance.main_applicant,
                                                                              sender: @user,
                                                                              key: NOTIFICATION_KEYS[:housing_request][:custom]
                                                                            }))

    if @notif.save
      render(
        jsonapi: @notif,
        serializer: ::Hermes::NotificationCenter::NotificationSerializer,
        status: :created
      )
    else
      render_json_error(:unprocessable_entity, :errors, {message: @notif.errors.full_messages})
    end
  end

  def send_message
    return render_json_error(:unauthorized, :not_als_collaborator) unless current_user_roles.include?('collaborateur offre logement')

    @doc = Doc.new
    @doc.assign_attributes(doc_params)
    @doc.type = 'email'
    @doc.responsible = @user.id
    @doc.bypass_refresh = true
    @doc.title = params[:name] if params[:name]
    @doc.title = params[:title] if params[:title]
    if @doc.save
      if to = params[:to] || (@doc.superfields && @doc.superfields['mail_to'])
        cc = params[:cc] || (@doc.superfields && @doc.superfields['mail_cc'])
        bcc = params[:bcc] || (@doc.superfields && @doc.superfields['mail_bcc'])
        from = 'no-reply@actionlogement.fr'
        if mail = Agent.regular_mail(@doc.id.to_s, to: to, title: @doc.title, from_no_reply: from, cc: cc, bcc: bcc, content: doc_params[:body])
          @doc.set(blob: mail.to_s, updated_at: DateTime.current)
          # @doc.get_email_infos

          mail.deliver_later
          render json: @doc, status: :created
        else
          render_json_error :unprocessable_entity, :sending_error
        end
      else
        render_json_error :unprocessable_entity, :missing_params_to
      end
    else
      respond_with_errors(@doc)
    end
  end

  def xml_sopra_errors
    return render_json_error(:not_found, :instance_not_found) unless @instance
    errors = Hermes::XmlSopraError.where(item_id: @instance.id, item_class: @instance.class).order_by(created_at: :desc)

    render(jsonapi: errors, each_serializer: Hermes::XmlSopraErrorSerializer)
  end 

  def config_change_minus_two
    render(json: { change_minus_two_date: HERMES_CONFIG['change_minus_two_date']&.to_s , change_minus_two_year: HERMES_CONFIG['change_minus_two_year']&.to_i }, status: 200)
  end 

  private

  def update_channel
    return render_json_error(:not_found, :instance_not_found) unless @instance

    if (params[:keys].present? && params[:keys].is_a?(Array)) || params[:key].present?
      yield if block_given?

      render json: {
        pending_channels: @instance.housing_channel_pending_keys,
        validated_channels: @instance.housing_channel_keys
      }
    else
      raise ActionController::ParameterMissing.new('key or keys')
    end

  rescue ActionController::ParameterMissing => e
    render_json_error(:not_acceptable, :parameter_missing, {message: e.message})
  rescue Hermes::Errors::HousingChannelError => e
    render_json_error(:unprocessable_entity, :housing_channel_error, {message: e.message})
  end


  def entity_code_validation(entity_code, applicant_id)
    if !entity_code.expired?
      applicant = Hermes::Applicant.find_by(id: applicant_id)
      return render_json_error(:not_found, :instance_not_found) unless applicant

      applicant.update!(entity_id: entity_code.entity_id)
      @instance.update!(last_entity_code: entity_code.code)
      render(jsonapi: @instance, serializer: @serializer)
    else
      render_json_error(:unprocessable_entity, :expire_code, {message: 'Le code est expiré.'})
    end
  end

  def doc_params
    if params.key?(:data)
      ActiveModelSerializers::Deserialization.jsonapi_parse!(params, only: %i[
        title
        responsible
        body
        cycle_id
        family_id
        type
        percent
        bloc_name
        doc_ids
        template_id
        note
        writers
        readers
        consulted
        accounted
        followed_by
        type
        tagset_ids
        space_ids
        ask_of
        pin
        tags
        labels
        superfields
        event_slot_type
        event_location
        event_modality
        event_slots
      ], key_transform: :unaltered)
    end
  end
end
