require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  ACCEPTABLE_PARAMS = %w[
    name
    email
    phone
    lessor_id
  ].freeze

  MUTATE_PARAMS = {
    name: 'String',
    email: 'String',
    phone: 'String',
    lessor_id: ['lessor1', 'lessor2']
  }.freeze

  describe Api::Dmo::LessorContactsController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    check_mutable_params(
      model: Hermes::LessorContact, 
      mutate_params: MUTATE_PARAMS, 
      factory: :lessor_contact,
      min_params: {
        name: 'test'
      },
      other_factory:[
        {factory: :lessor, id: 'lessor1'},
        {factory: :lessor, id: 'lessor2'},
      ],
    )

    let!(:responsible) { create(:user, roles: ['Administrator']) }
    let!(:user) { create(:user) }
    let!(:lessor) { create(:lessor, siret: '00000000000000', readers: [user.id.to_s]) }
    let!(:lessor2) { create(:lessor, siret: '00000000000000', readers: [responsible.id.to_s]) }
    
    let!(:lessor_contact1) { create(:lessor_contact, name: 'name1', lessor_id: lessor2.id.to_s) }
    let!(:lessor_contact2) { create(:lessor_contact, name: 'name2', lessor_id: lessor2.id.to_s) }
    let!(:lessor_contact3) { create(:lessor_contact, name: 'test3', lessor_id: lessor2.id.to_s) }
    let!(:lessor_contact4) { create(:lessor_contact, name: 'test4', lessor_id: lessor.id.to_s) }
    

    describe 'POST /lessor_contacts' do
      describe 'Create as admin' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                      name: 'test'
                  }
              }
          }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end
      end

      describe 'Create as user without lessor_id' do
        before :each do
          authenticated_header(request, user)
          post 'create', {
              data: {
                  attributes: {
                      name: 'test'
                  }
              }
          }
        end
        it 'returns error when no lessor_id' do
          expect(response).to have_http_status(422)
        end
      end

      describe 'Create as user with lessor_id' do
        before :each do
          authenticated_header(request, user)
          post 'create', {
              data: {
                  attributes: {
                      name: 'test',
                      lessor_id: lessor.id.to_s
                  }
              }
          }
        end
        it 'returns 201' do
          expect(response).to have_http_status(201)
        end
      end
    end

    describe 'GET /lessor_contacts' do
      describe 'as admin' do 
        before :each do
          authenticated_header(request, responsible)
          get 'index'
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(4)
        end
      end
      
      describe 'filter by lessor_id' do 
        before :each do
          authenticated_header(request, responsible)
          get 'index', {lessor_id: lessor2.id.to_s}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(3)
        end
      end
      describe 'as user' do 
        before :each do
          authenticated_header(request, user)
          get 'index'
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(1)
        end
      end

      describe 'regex on name' do 
        before :each do
          authenticated_header(request, responsible)
          get 'index', {name: {"$regex": "test" }}
        end
        it 'returns a valid json' do
          expect(response).to have_http_status(200)
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(2)
        end
      end
      
    end

    describe 'GET /lessor_contacts/:id' do
      describe 'verif it work' do
        before :each do
          authenticated_header(request, user)
          get 'show', {id: lessor_contact4.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end

      describe 'verif it work not work with a non authorized user' do

        before :each do
          authenticated_header(request, user)
          get 'show', {id: lessor_contact1.id}
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end
    end


    describe 'DELETE /lessor_contacts/:id' do
      before :each do
        authenticated_header(request, responsible)

        delete 'destroy', {
            id: lessor_contact1.id.to_s
        }
      end
      it 'returns status code 204' do
        expect(response).to have_http_status(204)
      end
    end

  end
end
