require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  ACCEPTABLE_PARAMS = %w[
    name
    code
    lessor_id
  ].freeze

  MUTATE_PARAMS = {
    user_id: ['user1', 'user2', 'user3'],
    key: ['key1', 'key2', 'key3'],
  }

  describe Api::Dmo::UserFiltersController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:user1) { create(:user) }
    let!(:user2) { create(:user) }
    

    describe 'POST /user_filters' do
      let!(:params) { { data: { attributes: { user_id: user1.id.to_s, key: 'key1', value: {test: 'bob', a: []} } } } }
      subject { post 'create', params }

      before :each do 
        authenticated_header(request, user1)
      end 
      
      it 'missing params' do 
        params[:data][:attributes][:user_id] = nil
        subject
        expect(response).to have_http_status(422)
        expect(json['errors'].first['title']).to match(/missing_params/)
      end 

      it "error when trying to create for other user" do 
        params[:data][:attributes][:user_id] = user2.id
        subject
        expect(response).to have_http_status(401)
        expect(json['errors'].first['title']).to match(/cannot_create_filter_for_other_user/)
      end 
      
      it 'create basic one' do
        subject
        expect(response).to have_http_status(201)
        expect(json['data']['attributes']['key']).to eq('key1')
        expect(json['data']['attributes']['value']["test"]).to eq('bob')
        expect(json['data']['attributes']['value']["a"]).to eq([])
      end

      it 'update existing key' do 
        uf = Hermes::UserFilter.create(user: user1, key: 'key1', value: { test: 'john'})
        subject
        expect(response).to have_http_status(201)
        expect(json['data']['attributes']['key']).to eq('key1')
        expect(json['data']['attributes']['value']["test"]).to eq('bob')
        expect(json['data']['id']).to eq(uf.id.to_s)
      end 
    end

    describe 'GET /user_filters' do
      let!(:uf1) { create(:user_filter, user: user1, key: 'key1') }
      let!(:uf2) { create(:user_filter, user: user1, key: 'key2') }
      let!(:uf3) { create(:user_filter, user: user1, key: 'key3') }
      let!(:uf4) { create(:user_filter, user: user2, key: 'key1') }

      it 'only return user filters' do
        authenticated_header(request, user1)
        get 'index'
        expect(response).to have_http_status(200)
        expect(json['data'].length).to eq(3)
      end
    end 

    describe 'GET /user_filters/:id' do
      let!(:uf1) { create(:user_filter, user: user1, key: 'key1') }
      
      it 'error not found' do
        authenticated_header(request, user1)
        get 'show', {id: 'unknown'}
        expect(response).to have_http_status(404)
        expect(json['errors'].first['title']).to match(/instance_not_found/)
      end

      it 'error when not user' do
        authenticated_header(request, user2)
        get 'show', {id: uf1.id}
        expect(response).to have_http_status(401)
        expect(json['errors'].first['title']).to match(/cannot_see_this_user_filter/)
      end

      it 'works' do
        authenticated_header(request, user1)
        get 'show', {id: uf1.id}
        expect(response).to have_http_status(200)
      end

    end

    describe 'DELETE /user_filters/:id' do
      let!(:uf1) { create(:user_filter, user: user1, key: 'key1') }
      
      it 'error not found' do
        authenticated_header(request, user1)
        delete 'destroy', {id: 'unknown'}
        expect(response).to have_http_status(404)
        expect(json['errors'].first['title']).to match(/instance_not_found/)
      end

      it 'error when not user' do
        authenticated_header(request, user2)
        delete 'destroy', {id: uf1.id}
        expect(response).to have_http_status(401)
        expect(json['errors'].first['title']).to match(/cannot_delete_other_user_filter/)
      end

      it 'works' do
        authenticated_header(request, user1)
        delete 'destroy', {id: uf1.id}
        expect(response).to have_http_status(204)
      end
    end

  end
end
